#if !defined(_IOU_THREAD_TEMPLATE_)
#define _IOU_THREAD_TEMPLATE_
// file		:	iouthread.h
// purpose	:	iou thread template
// (c) Allan Kelly 1998-1999

// System includes --------------------
#if !defined(_WINDOWS_)
#include <process.h>
#include <windows.h>
#endif

#include <cassert>
#include <stdexcept>

namespace Accu
{

template<class T, class R> 
class IouThread
{
	void*	_threadHandle;
	T		_worker;

public:
	IouThread(T object) : _threadHandle(NULL), _worker(object)
	{
	}

	~IouThread()
	{
		if (NULL != _threadHandle)
		{
			Complete();
		}
	}

	static unsigned int _stdcall StartRun(void *param)
	{	
		T* object = static_cast<T*> (param);	// reclaim object type
		object->Run();
		return 0;
	}

	void RunThread(	long stackSize = 0)
	{
		assert (NULL == _threadHandle);
		// doc says _beginthread return unsigned long, but -1 for error!
		//unsigned long h = _beginthread(StartRun, stackSize, );
		unsigned int threadId;
		unsigned long h = _beginthreadex(0, 0, StartRun, 
										&(_worker), 0, &threadId);

		if (0 == h)
		{
			throw std::runtime_error("Failed to start thread");
		}
		_threadHandle = reinterpret_cast<void*> (h);
	}

	bool IsTerminated()
	{
		if (NULL == _threadHandle)
		{	// not run yet so is terminated
			return true;
		}

		DWORD w = WaitForSingleObject(_threadHandle, 0);
		if (WAIT_OBJECT_0 == w)
		{
			return true;
		}

		return false;
	}

	void Complete()
	{
		if (!IsTerminated())
		{
			WaitForSingleObject(_threadHandle, INFINITE);
		}
	}

	R Redeem()
	{
		if (NULL == _threadHandle)
		{	// thread not run yet
			RunThread();
		}
		Complete();
		return _worker.Result();
	}
};  // IouThread

} // namespace
#endif
