#if !defined(_IOU_THREAD_TEMPLATE_)
#define _IOU_THREAD_TEMPLATE_
// file		:	iouthread2.h
// purpose	:	iou thread template
//				platform independent version
// (c) Allan Kelly 1998-1999

// System includes --------------------
#include <cassert>
#include <stdexcept>

// Project includes -------------------
#include "thdfuncs.h"

#if defined(NAME_SPACES)
namespace Accu
{
#endif

template<class T, class R> 
class IouThread
{
	ThreadTypes::ThreadId	_threadHandle;
	T						_worker;

public:
	IouThread(T object) : _threadHandle(0), _worker(object)
	{
	}

	~IouThread()
	{
		if (0 != _threadHandle)
		{
			Complete();
			DestroyThread(_threadHandle);
		}
	}

	static int StartRun(void *param)
	{	
		T* object = static_cast<T*> (param);	// reclaim object type
		object->Run();
		return 0;
	}

	void RunThread(	long stackSize = 0)
	{
		assert (0 == _threadHandle);
		_threadHandle = CreateThread(StartRun, &_worker);
	}

	void Complete()
	{
		WaitForThread(_threadHandle);
		DestroyThread(_threadHandle);
		_threadHandle = 0;
	}

	R Redeem()
	{
		if (0 == _threadHandle)
		{	// thread not run yet
			RunThread();
		}
		Complete();
		return _worker.Result();
	}
};  // IouThread

#if defined(NAME_SPACES)
} // namespace
#endif

#endif
