// file		:	exmaple.cpp
// purpose	:	example of Iou cross platform thread
// (c) Allan Kelly 1998-1999

// system includes ------------
#include <cstdlib>
#include <iostream>
#include <string>

// project includes -----------
#include "auxfuncs.h"
//#include "iouthrd.h"			// win32 only
#include "iouthrd2.h"		// Posix or Win32
#include "critical.h"

#if defined(NAME_SPACES)
using namespace Accu;
#endif

class Economy
{
	std::string			_place;
	int					_currentGdp;
	double				_growthRate;
	int					_years;
	CriticalSection&	_criticalSec;

public:
	typedef int ReturnType;

	Economy(const std::string& place,
			int currentGdp, 
			double monthlyGrowth, 
			int years,
			CriticalSection& guardSection) :
				_place(place),
				_currentGdp(currentGdp), 
				_growthRate(monthlyGrowth), 
				_years(years),
				_criticalSec(guardSection)
	{
	}

	Economy(const Economy& src) :
				_place(src._place),
				_currentGdp(src._currentGdp), 
				_growthRate(src._growthRate), 
				_years(src._years),
				_criticalSec(src._criticalSec)
	{
	}

	void Run()
	{
		double newGdp = _currentGdp;
		for (	int i = 1; i <= _years; i++)
		{
			for (int month = 0; month < 12; month++)
			{
				double r = static_cast<double> (rand());
				double wobbleFactor = (r / RAND_MAX) - 0.3;
				newGdp += (newGdp * _growthRate * wobbleFactor);
			}
			{
				// watch for GNU compiler optimisations!
				CriticalToken guard(_criticalSec);
				std::cout << "At the end of year " << i
						<< " in " << _place
						<< " GDP = " << newGdp << std::endl;
			}
			//SleepSeconds(1);  // try without me!
		}
		_currentGdp = static_cast<int> (newGdp);
	}

	ReturnType Result() { return _currentGdp; }
};

int main()
{
	try
	{
		CriticalSection outputGuard;
		const int years = 15;
		IouThread<Economy, Economy::ReturnType> 
			england(Economy("England", 1000, (0.3/12), years, outputGuard));
		IouThread<Economy, Economy::ReturnType>
			scotland(Economy("Scotland", 500, (0.4/12), years, outputGuard));
		IouThread<Economy, Economy::ReturnType>
			wales(Economy("Wales", 250, (0.25/12), years, outputGuard));
		england.RunThread();
		scotland.RunThread();
		wales.RunThread();

		std::cout << "After " << years << " years the UK economy has grown to "
					<< (england.Redeem() + scotland.Redeem() + wales.Redeem())
					<< std::endl;
	}  // try
	catch(std::exception& exp)
	{
		std::cerr << "Error occured:" << exp.what() << std::endl;
		return 1;
	}
	catch(...)
	{
		std::cerr << "Unknown error occured" << std::endl;
		return 2;
	}

	WaitForThreads();
	return 0;
}
