// file		:	thdfuncs.cxx
// purpose	:	iou thread template
// (c) Allan Kelly 1998-1999

// System includes --------------------
#if defined(WIN32)
#include <process.h>
#include <windows.h>
#else
#include <pthread.h>
#endif

#include <stdexcept>

// local includes ---------------------
#include "thdfuncs.h"

#if defined(WIN32)

// reopen namespace for one function we _do_not_ want in header file
namespace Accu
{
unsigned int _stdcall 
StartFunc(void *param)
{
	Accu::StartData* start = static_cast<Accu::StartData*>(param);
	int r = start->_startRun(start->_arg);
	delete start;
	return r;
}
} // namespace

Accu::ThreadTypes::ThreadId 
Accu::CreateThread(	ThreadTypes::StartRunFunc	addr,
					void*						arg)
{
	unsigned int threadId;
	StartData* start = new StartData(addr, arg);
	unsigned long id = _beginthreadex(0, 0, Accu::StartFunc, 
										start, 0, &threadId);
	if ( id == static_cast<unsigned long> (0))
	{
		throw std::runtime_error("Failed to create thread");
	}
	return id;
}

void Accu::WaitForThread(Accu::ThreadTypes::ThreadId id)
{
	DWORD r = WaitForSingleObject(reinterpret_cast<void*>(id), INFINITE);
	DWORD err = GetLastError();
	if (WAIT_FAILED == r)
	{
		throw std::runtime_error("Wait for thread failed");
	}
}

void Accu::DestroyThread(ThreadTypes::ThreadId id)
{
	CloseHandle(reinterpret_cast<void*>(id));
}

#else

void*
StartFunc(void *param)
{
	StartData* start = static_cast<StartData*>(param);
	int r = start->_startRun(start->_arg);
	delete start;
}

unsigned long CreateThread(	ThreadTypes::StartRunFunc	addr,
							void*						arg)
{
	pthread_t		id;
	pthread_attr_t	attributes;
	StartData* start = new StartData(addr, arg);
	int r = pthread_create(	&id, 
							NULL,
							StartFunc, 
							start);
	if (	(r == EINVAL)
		||	(r == EAGAIN) )
	{
		throw runtime_error("Failed to create thread");
	}

	return id;
}

void WaitForThread(ThreadTypes::ThreadId id)
{
	void* result;
	int r = pthread_join(id, &result);
	if (	(EINVAL == r)
		||	(ESRCH == r) )
	{
		throw std::runtime_error("Failed to join thread");
	}
}

void DestroyThread(ThreadTypes::ThreadId id)
{
	int r = pthread_detach(id);

	// r could = EINVAL or ESRCH
	// if ESRCH thread has probably ceased to exists anyway
	if (EINVAL == r)
	{
		throw std::runtime_error("Failed to detach thread");
	}
}


#endif

